@@ -2,16 +2,16 @@ import enum
 
 
 class Source(enum.Enum):
+    UNKNOWN = "Unknown"
+    TEMPLATE = "Template"
+
     CALLING = "Calling"
     DUNGEON = "Dungeon"
-    KORTHIA = "Korthia"
-    ZERETH_MORTIS = "Zereth Mortis"
     MISSION = "Mission"
     PROFESSION = "Profession"
     PVP = "PvP"
     RAID = "Raid"
     RARE_MOB = "Rare Mob"
-    UNKNOWN = "Unknown"
     WORLD_BOSS = "World Boss"
     WORLD_DROP = "World Drop"
     WORLD_QUEST = "World Quest"
